/*

<Aicaca Community Messenger Server>

Copyright (c) Aicaca.com, All rights reserved.

Writer                : Byung-ku, Cho (Aicaca)
E-mail                : aicaca@hanmail.net
Personal Homepage URL : http://www.aicaca.com
Last Updated Date     : Feb. 27th, 2002

Programming Language  : C#
Editor & Compiler     : VisualStudio.NET

*/


using System;
using System.IO;
using System.Threading;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Xml;

namespace TalkServer
{
    public class WinDialog
    {
		private WinCom WC = new WinCom();
		private WinServer xWinServer = null;
        private WinBoard xWinBoard = null;

		private Socket DialogSocket = null;
		private TextReader SocketReader = null;
		private TextWriter SocketWriter = null;
		
		private string rootID;
        private string clientID;
        private string threadID;
        
        public WinDialog(WinServer tWinServer, WinBoard tWinBoard, Socket tDialogSocket, string tRootID)
        {
            xWinServer = tWinServer;
			xWinBoard = tWinBoard;
			DialogSocket = tDialogSocket;
			rootID = tRootID;

			if (DialogSocket != null && DialogSocket.Connected)
			{
				NetworkStream stream = new NetworkStream(DialogSocket);
				SocketReader = new StreamReader(stream);
				SocketWriter = new StreamWriter(stream);
			}
        }

		public void CloseThis()
		{
			if (DialogSocket != null)
			{
				DialogSocket.Close();
				DialogSocket = null;
			}
		}

		private string SaveFile(string fileName, int fileLen, byte[] fileStream)
		{
			fileName = new FileInfo(fileName).Name;
            int num = 1;

			while (true)
			{
				FileInfo xFileInfo = new FileInfo(WC.varStr1 + "Upload\\" + fileName);

				if (xFileInfo.Exists == false)
				{
					FileStream fs = new FileStream(WC.varStr1 + "Upload\\" + fileName, FileMode.Create);
					fs.Write(fileStream,0,fileLen);
					fs.Close();
					break;
				}
				else
				{
					fileName = num.ToString() + fileName;
					num++;
				}
			}

			return fileName;
		}

		private void OnAddClient(string str)
		{
		    string clientSort = WC.GetXML(str, "ls");
		    string clientName = WC.GetXML(str, "ln");
		    string clientIP = WC.GetXML(str, "li");
		    string clientPort = WC.GetXML(str, "lp");
		    clientID = WC.MakeClientID(clientSort, clientName, clientIP, clientPort);
		    threadID = WC.MakeThreadID("WinDialog", clientName, clientIP, clientPort);

            xWinServer.AddClient(this, clientID, threadID);
		}

        private void OnRemoveClient()
		{
			xWinServer.RemoveClient(clientID, threadID);
		}

		private void OnRoundMsg(string str)
		{
			try
			{
			    string roundName = WC.GetXML(str, "tn");
			    string roundIP = WC.GetXML(str, "ti");
			    string roundPort = WC.GetXML(str, "tp");
			    string roundID = WC.MakeThreadID("WinDialog", roundName, roundIP, roundPort);
			    xWinServer.SendRoundMsg(roundID, str);
			}
			catch (Exception e)
			{
				string tempThreadID = WC.RemoteToThread(WC.GetXML(str, "eid"));
				string tempRemoteID = WC.ThreadToRemote(WC.GetXML(str, "tid"));
                string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + tempThreadID + tempRemoteID + WC.MakeXML("er", WC.lineStr4);
			    this.SendMsg(sendStr);
			}
		}

        private void OnRoundFile(string str)
		{
			try
			{
			    string roundName = WC.GetXML(str, "tn");
			    string roundIP = WC.GetXML(str, "ti");
			    string roundPort = WC.GetXML(str, "tp");
			    string roundID = WC.MakeThreadID("WinDialog", roundName, roundIP, roundPort);

                string fileName = WC.GetXML(str, "fn");
			    int fileLen = Convert.ToInt32(WC.GetXML(str, "fl"));
		        byte[] b = new byte[fileLen];

				int idx = 0;
				while (fileLen > idx)
				{
					idx = idx + DialogSocket.Receive(b, idx, fileLen-idx, 0);
				}

                xWinServer.SendRoundFile(roundID, str, b);
			}
			catch (Exception e)
			{
				string tempThreadID = WC.RemoteToThread(WC.GetXML(str, "eid"));
				string tempRemoteID = WC.ThreadToRemote(WC.GetXML(str, "tid"));
                string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + tempThreadID + tempRemoteID + WC.MakeXML("er", WC.lineStr4);
			    this.SendMsg(sendStr);
			}
		}

		private void OnRoundError(string str)
		{
			try
			{
				string roundName = WC.GetXML(str, "tn");
				string roundIP = WC.GetXML(str, "ti");
				string roundPort = WC.GetXML(str, "tp");
				string roundID = WC.MakeThreadID("WinDialog", roundName, roundIP, roundPort);
				xWinServer.SendRoundMsg(roundID, str);
			}
			catch (Exception e)
			{
			}
		}

		private void OnRoundChat(string str)
		{
			try
			{
				xWinServer.SendRoundChat(str);
			}
			catch (Exception e)
			{
			}
		}

		private void OnBoardList(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				byte[] sendStream = xWinBoard.ListProcess(sort);

				if (sendStream != null)
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
					this.SendMsg(sendStr);
					this.SendStream(sendStream);
				}
				else
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", "0");
					this.SendMsg(sendStr);
				}
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardMsg(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				string currentID = WC.GetXML(str, "xi");
				byte[] sendStream = xWinBoard.MsgProcess(sort, currentID);

				if (sendStream != null)
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXME") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
					this.SendMsg(sendStr);
					this.SendStream(sendStream);
				}
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardSave(string str)
		{
			try
			{
			    string sort = WC.GetXML(str, "xs");
				int len = Convert.ToInt32(WC.GetXML(str, "xl"));
				byte[] getStream = new byte[len];
				int idx = 0;

				while (len > idx)
				{
					idx = idx + DialogSocket.Receive(getStream, idx, len-idx, 0);
				}

				if (WC.GetXML(str, "xf").Length > 0)
				{
					string fileName = WC.GetXML(str, "xf");
					int fileLen = Convert.ToInt32(WC.GetXML(str, "xn"));
					byte[] fileStream = new Byte[fileLen];
					int fileIdx = 0;

					while (fileLen > fileIdx)
					{
						fileIdx = fileIdx + DialogSocket.Receive(fileStream, fileIdx, fileLen-fileIdx, 0);
					}

					fileName = this.SaveFile(fileName, fileLen, fileStream);
					xWinBoard.SaveProcess(sort, getStream, fileName);
				}
				else
				{
					xWinBoard.SaveProcess(sort, getStream, "N/A");
				}

                byte[] sendStream = xWinBoard.ListProcess(sort);
			    string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
			    this.SendMsg(sendStr);
				this.SendStream(sendStream);
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardModify(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				string currentID = WC.GetXML(str, "xi");
				int len = Convert.ToInt32(WC.GetXML(str, "xl"));
				byte[] getStream = new byte[len];
				int idx = 0;

				while (len > idx)
				{
					idx = idx + DialogSocket.Receive(getStream, idx, len-idx, 0);
				}

				xWinBoard.ModifyProcess(sort, currentID, getStream);

				byte[] sendStream = xWinBoard.ListProcess(sort);
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
				this.SendMsg(sendStr);
				this.SendStream(sendStream);
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardReply(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				string currentID = WC.GetXML(str, "xi");
				int len = Convert.ToInt32(WC.GetXML(str, "xl"));
				byte[] getStream = new byte[len];
				int idx = 0;

				while (len > idx)
				{
					idx = idx + DialogSocket.Receive(getStream, idx, len-idx, 0);
				}

				if (WC.GetXML(str, "xf").Length > 0)
				{
					string fileName = WC.GetXML(str, "xf");
					int fileLen = Convert.ToInt32(WC.GetXML(str, "xn"));
					byte[] fileStream = new Byte[fileLen];
					int fileIdx = 0;

					while (fileLen > fileIdx)
					{
						fileIdx = fileIdx + DialogSocket.Receive(fileStream, fileIdx, fileLen-fileIdx, 0);
					}

					fileName = this.SaveFile(fileName, fileLen, fileStream);
					xWinBoard.ReplyProcess(sort, currentID, getStream, fileName);
				}
				else
				{
					xWinBoard.ReplyProcess(sort, currentID, getStream, "N/A");
				}

				byte[] sendStream = xWinBoard.ListProcess(sort);
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
				this.SendMsg(sendStr);
				this.SendStream(sendStream);
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardDelete(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				string currentID = WC.GetXML(str, "xi");
				xWinBoard.DeleteProcess(sort, currentID);
				byte[] sendStream = xWinBoard.ListProcess(sort);

				if (sendStream != null)
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
					this.SendMsg(sendStr);
					this.SendStream(sendStream);
				}
				else
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", "0");
					this.SendMsg(sendStr);
				}
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardSearch(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				string type = WC.GetXML(str, "xt");
				string keyword = WC.GetXML(str, "xk");
				byte[] sendStream = xWinBoard.SearchProcess(sort, type, keyword);

				if (sendStream != null)
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", sendStream.Length.ToString());
					this.SendMsg(sendStr);
					this.SendStream(sendStream);
				}
				else
				{
					string sendStr = WC.MakeXML("ms", "ILOVEAICACAXLI") + rootID + clientID + WC.MakeXML("xs", sort) + WC.MakeXML("xl", "0");
					this.SendMsg(sendStr);
				}
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardFile(string str)
		{
			try
			{
				string originFileName = WC.GetXML(str, "xo");
				string changeFileName = WC.GetXML(str, "xc");

				FileStream fs = new FileStream(WC.varStr1 + "Upload\\" + originFileName, FileMode.Open);
				int fileLen = Convert.ToInt32(fs.Length);
				byte[] fileStream = new byte[fileLen];
				fs.Read(fileStream,0,fileLen);
				fs.Close();

				string sendStr = WC.MakeXML("ms", "ILOVEAICACAXFI") + rootID + clientID + WC.MakeXML("xc", changeFileName) + WC.MakeXML("xl", fileLen.ToString());
				this.SendMsg(sendStr);
				this.SendStream(fileStream);
			}
			catch (Exception e)
			{
				string sendStr = WC.MakeXML("ms", "ILOVEAICACAERR") + rootID + clientID + WC.MakeXML("er", WC.lineStr5);
				this.SendMsg(sendStr);
			}
		}

		private void OnBoardBatch(string str)
		{
			try
			{
				string sort = WC.GetXML(str, "xs");
				int len = Convert.ToInt32(WC.GetXML(str, "xl"));
				byte[] getStream = new byte[len];
				int idx = 0;

				while (len > idx)
				{
					idx = idx + DialogSocket.Receive(getStream, idx, len-idx, 0);
				}

				xWinBoard.SaveProcess(sort, getStream, "N/A");
			}
			catch (Exception e)
			{
				this.CloseThis();
			}
		}

		private void OnReceive(string str)
		{
		    str = str.Substring(0, str.Length-1);
            string sort = WC.GetXML(str, "ms");

			switch(sort)
			{
				case "ILOVEAICACACAR":
                    OnAddClient(str);
					break;
		        case "ILOVEAICACAMSG":
			        OnRoundMsg(str);
					break;
	            case "ILOVEAICACAFNO":
			        OnRoundMsg(str);
					break;
				case "ILOVEAICACAFCA":
					OnRoundMsg(str);
					break;
		        case "ILOVEAICACAFAC":
			        OnRoundMsg(str);
					break;
		        case "ILOVEAICACAFRE":
			        OnRoundMsg(str);
					break;
		        case "ILOVEAICACAFIL":
			        OnRoundFile(str);
					break;
				case "ILOVEAICACAERR":
					OnRoundError(str);
					break;
				case "ILOVEAICACACHA":
					OnRoundChat(str);
					break;
				case "ILOVEAICACABLI":
					OnBoardList(str);
					break;
				case "ILOVEAICACABSA":
					OnBoardSave(str);
					break;
				case "ILOVEAICACABME":
					OnBoardMsg(str);
					break;
				case "ILOVEAICACABDE":
					OnBoardDelete(str);
					break;
				case "ILOVEAICACABMO":
					OnBoardModify(str);
					break;
				case "ILOVEAICACABRE":
					OnBoardReply(str);
					break;
				case "ILOVEAICACABSE":
					OnBoardSearch(str);
					break;
				case "ILOVEAICACABFI":
					OnBoardFile(str);
					break;
				case "ILOVEAICACABAT":
					OnBoardBatch(str);
					break;
		    }
		}

		private void SendHttpMsg(string str)
		{
			FileStream fs = new FileStream(WC.varStr1 + "Root\\" + str, FileMode.Open);
			int fileLen = Convert.ToInt32(fs.Length);
			byte[] sendStream = new byte[fileLen];
			fs.Read(sendStream,0,fileLen);
			fs.Close();

			DialogSocket.Send(sendStream);

			if (DialogSocket != null)
			{
				DialogSocket.Close();
				DialogSocket = null;
			}
		}

		private void OnHttp(string httpStr)
		{
            string requestType = httpStr.Substring(0, 3);
			string requestFile = "";
			int firstPos;
			int secondPos;

			if (requestType == "GET")
			{
				firstPos = 5;
				secondPos = httpStr.IndexOf(" ", 5);
                requestFile = httpStr.Substring(firstPos, secondPos-firstPos);
				if (requestFile.Length == 0) requestFile = "index.html";
				this.SendHttpMsg(requestFile);
			}
			else
			{
				if (DialogSocket != null)
				{
					DialogSocket.Close();
					DialogSocket = null;
				}
			}
                
		}

		private void HttpProc(string httpStr)
		{
			string strOne = "";
			string str = httpStr;
			char[] c = new char[1];

			while (SocketReader.Read(c,0,1) != 0)
			{
				strOne = new String(c);
				str += strOne;
                
				if (str.Substring(str.Length-4, 4) == "\r\n\r\n")
				{
					this.OnHttp(str);
					break;
				}
			}
		}

		public void SendConfigData(string str)
		{
            string sendStr = WC.MakeXML("ms", "ILOVEAICACACFG") + rootID + clientID + str;
			this.SendMsg(sendStr);
		}

		public void SendClientAddr(string str)
		{
			string sendStr = WC.MakeXML("ms", "ILOVEAICACACAS") + rootID + clientID + WC.MakeXML("car", str);
			this.SendMsg(sendStr);
		}

		public void SendMsg(string str)
		{ 
			str = WC.ReplaceStr(str, "\n", "&Line;") + "\n";
			SocketWriter.Write(str);
			SocketWriter.Flush();
		}

		public void SendStream(byte[] b)
		{ 
			DialogSocket.Send(b, b.Length, 0);
		}

		public void SendChat(string str)
		{ 
			SocketWriter.Write(str + "\n");
			SocketWriter.Flush();
		}

		public void ServerProc()
		{
			string strOne = "";
            string str = "";
			char[] c = new char[1];

			try
			{
				while (SocketReader.Read(c,0,1) != 0)
				{
					strOne = new String(c);
					str += strOne;
					
					if (strOne == "\n")
					{
						lock(this)
						{
							if (str.IndexOf("HTTP") != -1)
							{
								this.HttpProc(str);
							}
							else if (str.IndexOf("ms") != -1)
							{
								this.OnReceive(str);
								str = "";
							}
							else
							{
								if (DialogSocket != null)
								{
									DialogSocket.Close();
									DialogSocket = null;
								}
							}
						}
					}	
				}
			}
			catch (Exception e)
			{
			}

			this.OnRemoveClient();

			if (DialogSocket != null)
			{
				DialogSocket.Close();
				DialogSocket = null;
			}
		}
    }
}
